#ifndef VertexReco_Vertex_H
#define VertexReco_Vertex_H
#include "DataFormats/Math/interface/Error.h" 
#include "DataFormats/Math/interface/Point3D.h" 
#include "DataFormats/TrackReco/interface/TrackFwd.h"

namespace reco {
  class Vertex {
    public:
      typedef std::vector<TrackBaseRef >::const_iterator trackRef_iterator;

      enum { dimension = 3 };
      typedef math::Error<dimension>::type CovarianceMatrix;
      typedef math::Error<dimension>::type Error;
      typedef math::XYZPoint Point;

      Vertex ( const Point &, const Error &, double chi2, double ndof, size_t size );
      Vertex ();
      virtual ~Vertex() {};

      double chi2() const;
      double ndof() const;
      double x() const;
      double y() const;
      double z() const;

      size_t tracksSize() const;
      void add ( const TrackRef &, double w=1.0 );
      void add ( const TrackRef &, const Track & refitted, double w=1.0 );
      trackRef_iterator tracks_begin() const;
      trackRef_iterator tracks_end() const;

      float trackWeight ( const TrackRef & r ) const;

      void removeTracks();

      const Point & position() const;
      CovarianceMatrix covariance() const;
    private:
      double chi2_;
      double ndof_;
      Point p_;
      CovarianceMatrix c_;
      int size_;
      std::vector < TrackRef > tracks_;
      std::vector < Track > refittedTracks_;
      std::vector<float> weights_;
  };
}

#endif
