#include "RaveBase/Converters/interface/RaveToPerigeeObjects.h"

#include "RaveBase/Converters/interface/RaveToCmsObjects.h"
#include "RaveBase/Converters/interface/CmsToRaveObjects.h"

namespace rave {

PerigeeMomentum3D RaveToPerigeeObjects::convert(
    const Vector3D & momentum, const Charge charge ) const
{
  /* This is done quite nasty by using the momentum information to build a 
   * state at the origin, which only works for homogenous magnetic fields.
   */

  Vector6D euclidianState( 0., 0., 0., momentum.x(), momentum.y(), momentum.z() );
  PerigeeState5D perigeeState = convert( euclidianState, charge );

  return PerigeeMomentum3D( perigeeState.omega(), perigeeState.tanLambda(), 
                            perigeeState.phi() );
}

PerigeeState5D RaveToPerigeeObjects::convert(
    const Vector6D & state, const Charge charge ) const
{
  RaveToCmsObjects frameworkInputConverter;
  FreeTrajectoryState fts( frameworkInputConverter.convert( state, charge ) );
  GlobalPoint referencePoint = frameworkInputConverter.convert( state.position() );

  PerigeeConversions parameterConverter;
  double transverseMomentum;
  PerigeeTrajectoryParameters ptp =
      parameterConverter.ftsToPerigeeParameters( fts, referencePoint, transverseMomentum );

  CmsToRaveObjects frameworkOutputConverter;
  return frameworkOutputConverter.convert( ptp );
}

PerigeeCovariance5D RaveToPerigeeObjects::convert(
    const Covariance6D & error, const Vector6D & state, const Charge charge) const
{
  RaveToCmsObjects frameworkInputConverter;
  FreeTrajectoryState fts( frameworkInputConverter.convert( state, charge ),
                           frameworkInputConverter.convert( error ) );
  GlobalPoint referencePoint = frameworkInputConverter.convert( state.position() );

  PerigeeConversions parameterConverter;
  double transverseMomentum;
  PerigeeTrajectoryParameters ptp =
      parameterConverter.ftsToPerigeeParameters( fts, referencePoint, transverseMomentum );
  PerigeeTrajectoryError pte =
      parameterConverter.ftsToPerigeeError( fts );

  CmsToRaveObjects frameworkOutputConverter;

  // It would be cheap to use the PerigeeTrajectoryParameters twice and return the
  // PerigeeState5D as well, but that could let SWIG stumble.
  return frameworkOutputConverter.convert( pte, ptp );
}

#ifdef WITH_KINEMATICS

PerigeeState6D RaveToPerigeeObjects::convert( const Vector7D &, const Charge ) const
{
  // TODO
  return PerigeeState6D();
}

PerigeeCovariance6D RaveToPerigeeObjects::convert(
  const Covariance7D & error, const Vector7D & state, const Charge charge ) const
{
  // TODO
  return PerigeeCovariance6D();
}

#endif

} // namespace rave
