#include "RaveBase/RaveEngine/interface/RaveVertexReconstructor.h"
#include "RaveBase/RaveEngine/interface/RaveParameterSetBuilder.h"
#include "RecoVertex/ConfigurableVertexReco/interface/ConfigurableVertexReconstructor.h"

using namespace std;

vector < TransientVertex > rave::VertexReconstructor::vertices
  ( const vector < reco::TransientTrack > & t ) const
{
  return theRector->vertices ( t );
}

vector < TransientVertex > rave::VertexReconstructor::vertices
  ( const vector < reco::TransientTrack > & t,
    const reco::BeamSpot & s ) const
{
  return theRector->vertices ( t, s );
}

vector < TransientVertex > rave::VertexReconstructor::vertices
  ( const vector < reco::TransientTrack > & prims,
    const vector < reco::TransientTrack > & secs,
    const reco::BeamSpot & s ) const
{
  return theRector->vertices ( prims, secs, s );
}

rave::VertexReconstructor::~VertexReconstructor()
{
  if ( theRector ) delete theRector;
}

rave::VertexReconstructor::VertexReconstructor ( const std::string & d ) :
  theRector ( 0 ), theMethod ( d )
{
  edm::ParameterSet set = rave::ParameterSetBuilder::create ( d );
  theRector = new ConfigurableVertexReconstructor ( set );
  if (!theRector ) 
  {
    cout << "[rave::VertexReconstructor] error: no reconstructor instantiated!"
         << endl;
  }
}

const string & rave::VertexReconstructor::method() const
{
  return theMethod;
}

rave::VertexReconstructor::VertexReconstructor ( const rave::VertexReconstructor & o ) :
  theRector ( 0 ), theMethod ( o.theMethod )
{
  if ( o.theRector ) theRector=o.theRector->clone();
}

rave::VertexReconstructor * rave::VertexReconstructor::clone() const
{
  return new rave::VertexReconstructor ( *this );
}
