//
// C++ Interface: KinematicParticle
//
// Description:
//
//
// Author: Fabian Moser <fabiamos@gmail.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef RAVEKINEMATICPARTICLE_H
#define RAVEKINEMATICPARTICLE_H

#include <rave/BasicKinematicParticle.h>

namespace rave
{

/**
 @author Fabian Moser <fabiamos@gmail.com>
*/
class RaveDllExport KinematicParticle :
      public BasicKinematicParticle::Proxy,
      public boost::less_than_comparable< KinematicParticle >
{
  protected:
    typedef BasicKinematicParticle::Proxy Base;

  public:
    KinematicParticle ( const rave::BasicKinematicParticle & );
    KinematicParticle();

    /**
     * The fullstate represents the 7-dimensional tuple including position,
     * momentum and mass of the particle.
     * @return 7-dimensional state vector
     */
    const rave::Vector7D & fullstate() const;

    /**
     * The fullerror represents the 7x7 error matrix for the state given by
     * fullstate().
     * @return 7x7 error matrix
     */
    const rave::Covariance7D & fullerror() const;

    /**
     * The state method inherited from AbstractTrack gives the 6-dimensional
     * tuple including position and energy. To obtain the mass together with
     * position and momentum, use fullstate(). This really gives the
     * full state anyway and is simply an upcast.
     * @return upcast 7-dimensional state vector
     */
    const rave::Vector6D & state() const;

    /**
     * As with state(), this gives the same as fullerror() since Covariance7D
     * is a subclass of Covariance6D.
     * @return upcast 7x7 error matrix
     */
    const rave::Covariance6D & error() const;

    Charge charge() const;

    const PerigeeState5D & perigeeState() const;
    const PerigeeCovariance5D & perigeeError() const;

    float chi2() const;
    float ndof() const;

    /**
     * @return The magnetic field assigned upon creation of this particle
     */
    boost::shared_ptr< rave::MagneticField > magneticField() const;

    /**
     * @return The pointer to the kinematic tree (if any) current particle
     * belongs to 0 pointer returned in case  not any tree is built yet
     */
//    KinematicTree correspondingTree() const;

    /**
     * @return The last constraint aplied to this particle.
     */
    boost::shared_ptr< rave::KinematicConstraint > lastConstraint() const;

    int id() const;
    boost::any link() const;
    std::string tag() const;

    bool isValid() const;

    bool operator< ( const rave::KinematicParticle & ) const;

};

}

#endif
