#ifndef _RavePerigeeCovariance3D_H_
#define _RavePerigeeCovariance3D_H_

namespace rave
{

/**
 * @class PerigeeCovariance3D stores covariance for perigee momentum
 *
 * This data class holds error and covariance information for a corresponding
 * PerigeeMomentum3D structure.
 *
 * The three parameters in the referred class are called omega, tanLambda and 
 * phi. To keep the argument names short here, they are abbreviated by their 
 * first character here.
 */

class RaveDllExport PerigeeCovariance3D
{
  public:
    PerigeeCovariance3D ( float doo, float dot, float dop,
                          float dtt, float dtp, float dpp );
    PerigeeCovariance3D();

    float doo() const; /// d omega * d omega
    float dot() const; /// d omega * d tanLambda
    float dop() const; /// d omega * d phi
    float dtt() const;
    float dtp() const;
    float dpp() const;

  private:
    float thedoo;
    float thedot;
    float thedop;
    float thedtt;
    float thedtp;
    float thedpp;
};

} // namespace rave

#endif
