#ifndef _RavePerigeeCovariance5D_H_
#define _RavePerigeeCovariance5D_H_

#include <rave/PerigeeCovariance3D.h>

namespace rave
{

/**
 * @class PerigeeCovariance5D stores covariance for perigee track parameters
 *
 * This data class holds error and covariance information for a corresponding
 * PerigeeState5D structure.
 *
 * The five parameters in the referred class are called omega, tanLambda, phi,
 * d0 and z0. To keep the argument names short here, they are abbreviated by
 * their first character here.
 */

class RaveDllExport PerigeeCovariance5D :
    public PerigeeCovariance3D
{
  public:
    PerigeeCovariance5D ( float doo, float dot, float dop, float dod,
                          float doz, float dtt, float dtp, float dtd,
                          float dtz, float dpp, float dpd, float dpz,
                          float ddd, float ddz, float dzz );
    PerigeeCovariance5D();

    float dod() const;
    float doz() const;
    float dtd() const;
    float dtz() const;
    float dpd() const;
    float dpz() const;
    float ddd() const;
    float ddz() const;
    float dzz() const;

  private:
    float thedod;
    float thedoz;
    float thedtd;
    float thedtz;
    float thedpd;
    float thedpz;
    float theddd;
    float theddz;
    float thedzz;
};

} // namespace rave

#endif
