#ifndef _RavePerigeeMomentum3D_H_
#define _RavePerigeeMomentum3D_H_

#include <cmath>

#include "boost/operators.hpp"

namespace rave
{

/**
 * @class PerigeeMomentum3D stores momentum-parameters only
 *
 * This data class manages a three-dimensional momentum vector in perigee
 * representation.
 *
 * The choice of parameters matches the L3 convention also used in LCIO files
 * see T.Kraemer LC-DET-2006-004
 */

class RaveDllExport PerigeeMomentum3D :
    boost::equality_comparable< PerigeeMomentum3D >
{
  public:
    PerigeeMomentum3D( float omega, float tanLambda, float phi );
    PerigeeMomentum3D();

    float omega() const;
    float tanLambda() const;
    float phi() const;

    bool equal( const PerigeeMomentum3D & other ) const;

  private:
    float theOmega;
    float theTanLambda;
    float thePhi;
};

RaveDllExport bool operator== ( const rave::PerigeeMomentum3D & lhs,
                                const rave::PerigeeMomentum3D & rhs );

} // namespace rave

#endif

