#ifndef _RaveTrack_H_
#define _RaveTrack_H_

#include <rave/Vector6D.h>
#include <rave/Covariance6D.h>
#include <rave/Charge.h>
#include <rave/BasicTrack.h>

#include <string>

namespace rave
{

/** \class RaveTrack defines the input to the rave library.
 * Fixme, it's more something like a RaveTrajectoryState,
 * than a RaveTrack.
 */

class RaveDllExport Track : private rave::BasicTrack::Proxy
{
  typedef rave::BasicTrack::Proxy Base;
  public:

    /**
     *  Create a RaveTrack from a 6-dimensional euclidean trajectory state
     *  plus covariance matrix, plus charge, plus an optional
     *  pointer to the "original track", whatever that means.
     *
     * lengths are given in cm, momenta in GeV/c.
     */
    Track( const rave::Vector6D &, const rave::Covariance6D &, Charge,
           float chi2, float ndof,
           void * originaltrack = 0, std::string tag="" );
    Track( int id, const rave::Vector6D &, const rave::Covariance6D &, Charge,
           float chi2, float ndof,
           void * originaltrack = 0, std::string tag="" );
    Track( const rave::Vector6D &, const rave::Covariance6D &, rave::Charge,
           float chi2, float ndof,
           int originaltrack, std::string tag="" );

    Track ( const rave::BasicTrack & );
    Track();

    rave::Charge charge() const;
    const rave::Vector6D & state() const;
    const rave::Covariance6D & error() const;
    void * originalObject() const;
    std::string tag() const;
    int id() const;

    float chi2() const;
    float ndof() const;

    const rave::PerigeeState5D & perigeeState() const;
    const rave::PerigeeCovariance5D & perigeeError() const;

    bool isValid() const;
    std::vector < rave::Track > components() const;

    bool operator< ( const rave::Track & ) const;
    bool operator== ( const rave::Track & ) const;
};

}

#endif
