//
// C++ Interface: TransientTrackKinematicParticle
//
// Description:
//
//
// Author: Fabian Moser <fabiamos@gmail.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef RAVETRANSIENTTRACKKINEMATICPARTICLE_H
#define RAVETRANSIENTTRACKKINEMATICPARTICLE_H

#include <rave/KinematicParticle.h>
#include <rave/BasicTransientTrackKinematicParticle.h>

namespace rave
{

/**
 * @class TransientTrackKinematicParticle is the kinematic input data structure
 *
 * Creating instances of TransientTrackKinematicParticle is the only way to 
 * generate the data members required by the KinematicTreeFactory to use
 * Rave's kinematic fitting possibilities.
 *
 * @author Fabian Moser <fabiamos@gmail.com>
 */

class RaveDllExport TransientTrackKinematicParticle :
      public KinematicParticle
{
  public:
    TransientTrackKinematicParticle (
      const rave::Track & initialTrack, const float & massGuess,
      const float & massSigma, const float & chiSquared,
      const float & degreesOfFr, boost::any origin = boost::any(),
      std::string tag = "" );

    TransientTrackKinematicParticle (
      const rave::Track & initialTrack, const float & massGuess,
      const float & massSigma, boost::any origin = boost::any(),
      std::string tag = "" );

    TransientTrackKinematicParticle (
      const rave::Vector7D & state, const rave::Covariance7D & error,
      const rave::Charge & charge, const float & chiSquared,
      const float & degreesOfFr, boost::any origin = boost::any(),
      std::string tag = "" );

    TransientTrackKinematicParticle (
      const rave::Vector7D & state, const rave::PerigeeCovariance6D & error,
      const rave::Charge & charge, const double & chiSquared,
      const double & degreesOfFr, boost::any origin = boost::any(),
      std::string tag = "" );

    TransientTrackKinematicParticle (
      const rave::PerigeeState6D & state, const rave::PerigeeCovariance6D & error,
      const rave::Charge & charge, const double & chiSquared,
      const double & degreesOfFr, boost::any origin = boost::any(),
      std::string tag = "" );

    TransientTrackKinematicParticle ( const rave::BasicTransientTrackKinematicParticle & );
    TransientTrackKinematicParticle();

};

}

#endif
