#include "RaveBase/Converters/interface/RaveToCmsObjects.h"
#include "RaveBase/Converters/interface/RaveToPerigeeObjects.h"
#include "RaveBase/RaveInterface/rave/BasicTrack.h"
#include "RaveBase/RaveEngine/interface/RaveId.h"

#include <iostream>

using namespace rave;

BasicTrack::BasicTrack( const Vector6D & s, const Covariance6D & e, Charge q,
   float chi2, float ndof, void * p, std::string d )
  : theState ( s ), theError ( e ), theCharge ( q ), thePointer ( p ),
    theTag ( d ), theId ( RaveId::uniqueId() ), theIsValid ( true ),
    theChi2( chi2 ), theNdof ( ndof ),
    hasCachedPerigeeRepresentation( false )
{
  // theComponents.push_back ( *this );
}

BasicTrack::BasicTrack( int id, const Vector6D & s, const Covariance6D & e, Charge q,
   float chi2, float ndof, void * p, std::string d )
  : theState ( s ), theError ( e ), theCharge ( q ), thePointer ( p ),
    theTag ( d ), theId ( id ), theIsValid ( true ),
    theChi2( chi2 ), theNdof ( ndof ),
    hasCachedPerigeeRepresentation( false )
{
  // theComponents.push_back ( *this );
}


float BasicTrack::chi2() const
{
  return theChi2;
}

float BasicTrack::ndof() const
{
  return theNdof;
}

std::string BasicTrack::tag() const
{
  return theTag;
}


bool BasicTrack::isValid() const
{
  return theIsValid;
}

Charge BasicTrack::charge() const
{
  return theCharge;
}

int BasicTrack::id() const
{
  return theId;
}

const Vector6D & BasicTrack::state() const
{
  return theState;
}

const Covariance6D & BasicTrack::error() const
{
  return theError;
}

void * BasicTrack::originalObject() const
{
  return thePointer;
}

const PerigeeState5D & BasicTrack::perigeeState() const
{
  if (!hasCachedPerigeeRepresentation) calculateCachedPerigeeRepresentation();
  return cachedPerigeeState;
}

const PerigeeCovariance5D & rave::BasicTrack::perigeeError() const
{
  if (!hasCachedPerigeeRepresentation) calculateCachedPerigeeRepresentation();
  return cachedPerigeeError;
}

void BasicTrack::calculateCachedPerigeeRepresentation() const
{
  RaveToPerigeeObjects converter;

  cachedPerigeeState = converter.convert( state(), charge() );
  cachedPerigeeError = converter.convert( error(), state(), charge() );

  hasCachedPerigeeRepresentation = true;
}

BasicTrack * BasicTrack::clone() const
{
  return new BasicTrack ( *this );
}

BasicTrack::BasicTrack ( const std::vector < BasicTrack > & components ) :
  theIsValid ( true ), theComponents ( components )
{
  // FIXME compute weighted mean
  std::cout << "[BasicTrack] error, missing implementation for multi-component track."
            << std::endl;
}

BasicTrack::BasicTrack () : theId(-1), theIsValid ( false )
{}

std::vector < BasicTrack > BasicTrack::components() const
{
  return theComponents;
}

bool BasicTrack::operator< ( const BasicTrack & o ) const
{
    return ( id() < o.id() );
}
