#ifndef KalmanVertexTrackUpdator_H
#define KalmanVertexTrackUpdator_H

#include "RecoVertex/VertexPrimitives/interface/VertexTrackUpdator.h"
#include "RecoVertex/VertexPrimitives/interface/CachingVertex.h"
#include "RecoVertex/VertexPrimitives/interface/VertexTrack.h"
#include "RecoVertex/VertexTools/interface/VertexTrackFactory.h"
#include "RecoVertex/KalmanVertexFit/interface/KVFHelper.h"
#include "RecoVertex/KalmanVertexFit/interface/KalmanVertexUpdator.h"

/** \class KalmanVertexTrackUpdator
 *  Performs the refit of the tracks with the vertex constraint, 
 *  using the Kalman filter algorithms.
 */

template <unsigned int N>
class KalmanVertexTrackUpdator : public VertexTrackUpdator<N> {

public:

  typedef typename CachingVertex<N>::RefCountedVertexTrack RefCountedVertexTrack;
  typedef typename VertexTrack<N>::RefCountedLinearizedTrackState RefCountedLinearizedTrackState;
  typedef typename VertexTrack<N>::RefCountedRefittedTrackState RefCountedRefittedTrackState;


  /**
   *  Default constructor
   */

  KalmanVertexTrackUpdator(){}

  virtual ~KalmanVertexTrackUpdator(){}

  /**
   *   Refit of the track with the vertex constraint.
   *   \param vertex The vertex which has to be used as constraint.
   *   \param track  The track to refit.
   *   \return	The VertexTrack containing the refitted track and 
   *		the track-to-vertex covariance.
   */

  RefCountedVertexTrack update(const CachingVertex<N> & vertex,
                               RefCountedVertexTrack track) const;


  /**
   *  Clone method
   */

  KalmanVertexTrackUpdator<N> * clone() const
  {
    return new KalmanVertexTrackUpdator(*this);
  }

  typedef ROOT::Math::SMatrix<double,3,N-2,ROOT::Math::MatRepStd<double,3,N-2> > AlgebraicMatrix3M;
  typedef std::pair< RefCountedRefittedTrackState, AlgebraicMatrix3M > trackMatrixPair; 

  /**
   *   Refit of the track with the vertex constraint.
   *   \param vertex The vertex which has to be used as constraint.
   *   \param track  The track to refit.
   *   \return	The refitted state with the track-to-vertex covariance.
   */

  trackMatrixPair trackRefit(const VertexState & vertex,
		RefCountedLinearizedTrackState linTrackState) const;

private:

  VertexTrackFactory<N> theVTFactory;
  KVFHelper<N> helper;
  KalmanVertexUpdator<N> updator;
};

#endif
