#include "TrackingTools/TransientTrack/interface/TransientTrack.h"
#include "TrackingTools/PatternTools/interface/TransverseImpactPointExtrapolator.h"
#include "DataFormats/GeometryCommonDetAlgo/interface/PerpendicularBoundPlaneBuilder.h"
#include "TrackingTools/PatternTools/interface/TSCPBuilderNoMaterial.h"
#include "RaveTools/Converters/interface/MagneticFieldSingleton.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"
#include <iostream>

using namespace reco;
using namespace std;

TransientTrack::TransientTrack( const FreeTrajectoryState & f, const GlobalPoint & gp,
   int id, float chi2, float ndf, void * rtrk ) :
    fts( f ),                          
    refPoint( gp ),
    stateAtVertexAvailable ( false ), theId(id), theChi2( chi2 ), theNdof ( ndf ),
    theRTrack(rtrk)
{
  /*
  if ( f.parameters().field()==0 ) 
  {
    std::cout << "[TransientTrack constor] FATAL: fts with no field!" << std::endl;
    exit(0);
  }
  */
}

TransientTrack::TransientTrack() :
  stateAtVertexAvailable ( false ), theId(-1), theChi2(-23. ), theNdof ( -23. ),
  theRTrack(0)
{
  // cout << "[TransientTrack default contor] ?!?" << endl; 
}

TransientTrack::TransientTrack( const TransientTrack & tt ) :
    fts( tt.fts ), refPoint( tt.refPoint ),
    // stateAtVertexAvailable ( false ),
    stateAtVertexAvailable ( tt.stateAtVertexAvailable ),
    theStateAtVertex ( tt.theStateAtVertex ),
    theId ( tt.theId ), theChi2 ( tt.theChi2 ), theNdof ( tt.theNdof ),
    theRTrack ( tt.theRTrack )
{
  // calculateStateAtVertex();
  if ( tt.id() == -1 )
  {
    cout << "[TransientTrack] trying to clone trk " << tt.id() << endl;
    std::cout << "[TransientTrack copy constor] FATAL: fts with no field!" << std::endl;
    // std::cout << "  -- old track had field " << tt.fts.parameters().field() << std::endl;
    exit(0);
  }
}

double TransientTrack::chi2() const
{
  return theChi2;
}

double TransientTrack::ndof() const
{
  return theNdof;
}

int TransientTrack::id() const
{
  return theId;
}

const TrackCharge TransientTrack::charge() const
{
    return fts.parameters().charge();
}

bool TransientTrack::operator== ( const TransientTrack & other ) const
{
    // return ( ( fts == other.fts ) && ( refPoint == other.refPoint ) );
    return ( id() == other.id() );
}    

bool TransientTrack::operator< ( const TransientTrack & other ) const
{
    return ( id() < other.id() );
}     

TrajectoryStateOnSurface TransientTrack::impactPointState() const
{
    if ( !stateAtVertexAvailable )
        calculateStateAtVertex();
        
    return theStateAtVertex;
}

TrajectoryStateOnSurface TransientTrack::innermostMeasurementState() const
{
  BoundPlane * p = PerpendicularBoundPlaneBuilder() ( fts.position(), fts.momentum() );
  return TrajectoryStateOnSurface ( fts, *p );
}

TrajectoryStateOnSurface TransientTrack::outermostMeasurementState() const
{
  // AHEM ....
  return innermostMeasurementState();
}

TrajectoryStateClosestToPoint TransientTrack::trajectoryStateClosestToPoint( 
        const GlobalPoint & point ) const
{
  TSCPBuilderNoMaterial builder;
  return builder(fts, point);
}

TrajectoryStateClosestToPoint TransientTrack::impactPointTSCP() const
{
  edm::LogWarning( "Kinematics" ) << "Not the true impactPoint!";
  return trajectoryStateClosestToPoint( GlobalPoint(0,0,0) );
}

void TransientTrack::calculateStateAtVertex() const
{
    theStateAtVertex = TransverseImpactPointExtrapolator( MagneticFieldSingleton::Instance() ).extrapolate( 
                           fts, refPoint ); // , MagneticFieldSingleton::Instance() );
    /* std::cout << "[TransientTrack] state at vertex. has error=" << theStateAtVertex.hasError()
         << std::endl;*/
    
    stateAtVertexAvailable = true;
}

const MagneticField * TransientTrack::field() const
{
  return MagneticFieldSingleton::Instance();
}

double TransientTrack::pt() const
{
  // FIXME check this, it has not appeared (yet?) in CMSSW
  return fts.momentum().perp();
}

float TransientTrack::px() const 
{
  return fts.momentum().x();
}

float TransientTrack::py() const 
{
  return fts.momentum().y();
}

float TransientTrack::pz() const 
{
  return fts.momentum().z();
}

const TransientTrack & TransientTrack::track() const
{
  return (*this);
}

const TransientTrack * TransientTrack::trackBaseRef() const
{
  return this;
}

void * TransientTrack::raveTrack() const
{
  return theRTrack;
}
